/*
 * Zorin Menu: The official applications menu for Zorin OS.
 *
 * Copyright (C) 2016-2020 Zorin OS Technologies Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Credits:
 * This file is based on code from the ArcMenu extension by
 * LinxGem33, Andrew Zaech, and Alexander Rüedlinger
 */

// Import Libraries
const Lang = imports.lang;
const Main = imports.ui.main;
const Gio = imports.gi.Gio;
const GObject = imports.gi.GObject;
const Shell = imports.gi.Shell;


// Local constants
const MUTTER_SCHEMA = 'org.gnome.mutter';
const WM_KEYBINDINGS_SCHEMA = 'org.gnome.desktop.wm.keybindings';

// The Menu Keybinder class manages keybindings for the menu
var MenuKeybinder = new Lang.Class({
    Name: 'MenuKeybinder',

    _init: function(menuToggler) {
        this._menuToggler = menuToggler;
        this.hotKeyEnabled = false;
        this.overlayKeyID = 0;
        this.defaultOverlayKeyID = 0;
        this.fromZorinMenu = false;
        this._mutterSettings = new Gio.Settings({ 'schema': MUTTER_SCHEMA });
        this._wmKeybindings = new Gio.Settings({ 'schema': WM_KEYBINDINGS_SCHEMA });
        this._oldPanelMainMenuKey = this._wmKeybindings.get_value('panel-main-menu');
        this._oldOverlayKey = this._mutterSettings.get_value('overlay-key');
        this.overlayKeyConnectID = this._mutterSettings.connect('changed::overlay-key', () => {
            if(!this.fromZorinMenu)
                this._oldOverlayKey = this._mutterSettings.get_value('overlay-key');
        });
        this.panelMainMenuKeyConnectID = this._wmKeybindings.connect('changed::panel-main-menu', () => {
            if(!this.fromZorinMenu)
                this._oldPanelMainMenuKey = this._wmKeybindings.get_value('panel-main-menu');
        });
        this._hotkeyMenuToggleId = Main.layoutManager.connect('startup-complete', ()=>{
            this._updateHotkeyMenuToggle();
        });
    },

    // Set Main.overview.toggle to toggle Zorin Menu instead
    enableHotKey: function() {
        this.fromZorinMenu = true;
        this._mutterSettings.set_string('overlay-key', 'SUPER_L');
        Main.wm.allowKeybinding('overlay-key', Shell.ActionMode.NORMAL |
            Shell.ActionMode.OVERVIEW | Shell.ActionMode.POPUP);
        this.hotKeyEnabled =  true;
        if(!Main.layoutManager._startingUp)
            this._updateHotkeyMenuToggle();
        this.fromZorinMenu = false;
    },

    // Set Main.overview.toggle to default function and default hotkey
    disableHotKey: function() {
        this.fromZorinMenu = true;
        this._mutterSettings.set_value('overlay-key', this._oldOverlayKey);
        if(this.overlayKeyID > 0){
            global.display.disconnect(this.overlayKeyID);
            this.overlayKeyID = null;
        }
        if(this.defaultOverlayKeyID>0){
            GObject.signal_handler_unblock(global.display, this.defaultOverlayKeyID);
            this.defaultOverlayKeyID = null;
        }
        Main.wm.allowKeybinding('overlay-key', Shell.ActionMode.NORMAL |
            Shell.ActionMode.OVERVIEW);
        this.hotKeyEnabled = false;
    
        this._wmKeybindings.set_value('panel-main-menu', this._oldPanelMainMenuKey);    
        this.fromZorinMenu = false;
        
    },

    // Update hotkey menu toggle function
    _updateHotkeyMenuToggle: function() {
        if(this.hotKeyEnabled){
            Main.wm.allowKeybinding('overlay-key', Shell.ActionMode.NORMAL |
            Shell.ActionMode.OVERVIEW | Shell.ActionMode.POPUP);

            //Find signal ID in Main.js that connects 'overlay-key' to global.display and toggles Main.overview
            let [bool,signal_id, detail] = GObject.signal_parse_name('overlay-key', global.display, true);
            this.defaultOverlayKeyID = GObject.signal_handler_find(global.display, GObject.SignalMatchType.ID, signal_id, detail, null, null, null); 

            //If signal ID found, block it and connect new 'overlay-key' to toggle Zorin Menu.
            if(this.defaultOverlayKeyID>0){
                GObject.signal_handler_block(global.display, this.defaultOverlayKeyID);
                this.overlayKeyID = global.display.connect('overlay-key', () => {
                    this._menuToggler();
                });
            }
            else
                global.log("Zorin Menu error: Failed to set Super_L hotkey");
        }   
        Main.wm.setCustomKeybindingHandler('panel-main-menu',
            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW | Shell.ActionMode.POPUP,
            this._menuToggler.bind(this));
    },

    // Destroy this object
    destroy: function() {
        // Clean up and restore the default behaviour
        if(this.overlayKeyConnectID){
            this._mutterSettings.disconnect(this.overlayKeyConnectID);
            this.overlayKeyConnectID = null;
        }
        if(this.panelMainMenuKeyConnectID){
            this._wmKeybindings.disconnect(this.panelMainMenuKeyConnectID);
            this.panelMainMenuKeyConnectID = null;
        }
        this.disableHotKey();
        if (this._hotkeyMenuToggleId) {
            // Disconnect the keybinding handler
            Main.layoutManager.disconnect(this._hotkeyMenuToggleId);
            this._hotkeyMenuToggleId = null;
        }
    }
});
