/*
 * Zorin Menu: The official applications menu for Zorin OS.
 *
 * Copyright (C) 2016-2019 Zorin OS Technologies Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Import Libraries
const Atk = imports.gi.Atk;
const GMenu = imports.gi.GMenu;
const Lang = imports.lang;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
const _ = Gettext.gettext;
const Widgets = Me.imports.widgets;
const Layouts = Me.imports.layouts;
const AppsBackend = Me.imports.appsbackend;
const Keybinder = Me.imports.keybinder;
const Desktop = Me.imports.desktop;

const ALL_ANIMATIONS = ~0;
const MENU_HEIGHT = 496;
const MENU_HEIGHT_OFFSET = 7;
const LAYOUTS = Layouts.layouts;

// Application menu class
var ApplicationsMenu = new Lang.Class({
    Name: 'ApplicationsMenu',
    Extends: PopupMenu.PopupMenu,

    // Initialize the menu
    _init: function(sourceActor, arrowAlignment, arrowSide, settings) {
        this.parent(sourceActor, arrowAlignment, arrowSide);
        this._section = new PopupMenu.PopupMenuSection();
        this.addMenuItem(this._section);
        this._settings = settings;
        this._appsBackend = new AppsBackend.AppsBackend();
        this._loadLayout();

        this._settingsId = this._settings.connect('changed::layout', Lang.bind(this, function() {
            this._reloadLayout();
        }));
        this.actor.connect('destroy', this._onDestroy.bind(this));
    },

    _loadLayout: function() {
        this._layout = Layouts.getLayout(this._settings, this._appsBackend);
        this._section.actor.add_child(this._layout.actor);
        this._layout.connect('activated', Lang.bind(this, function() {
            this._onLayoutActivated();
        }));
    },

    _reloadLayout: function() {
        this._layout.destroy();
        this._loadLayout();
    },

    // Return that the menu is not empty (used by parent class)
    isEmpty: function() {
	    return false;
    },

    // Handle opening the menu
    open: function(animate) {
        this.parent(animate);
        this._layout.reset();
        this._updateHeight();
    },

    _updateHeight: function() {
        let [minHeight, naturalHeight] = this._getPreferredHeight();
        this._layout.actor.set_height(minHeight);
    },

    _getPreferredHeight: function() {
        let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
        let layoutSetting = this._settings.get_enum('layout');
            switch(layoutSetting) {
                case LAYOUTS.ALL:
                    let [minHeight, naturalHeight] = this._layout._rightBox.get_preferred_height(-1);
                    let offset = MENU_HEIGHT_OFFSET * scaleFactor;
                    return [minHeight - offset, naturalHeight - offset];
                case LAYOUTS.SYSTEM_ONLY:
                    return this._layout._box.get_preferred_height(-1);
                default:
                    let height = MENU_HEIGHT * scaleFactor;
                    return [height, height];
        }
    },

    // Handle menu item activation
    _onLayoutActivated: function() {
        this.close(ALL_ANIMATIONS)
        if (Main.overview.visible)
            Main.overview.hide();
    },

    // Handle closing the menu
    close: function(animate) {
        this.parent(animate);
    },

    // Toggle menu open state
    toggle: function() {
        this.parent();
    },

    _onDestroy: function() {
        this._appsBackend.destroy();
        this._settings.disconnect(this._settingsId);
    }
});

// Application Menu Button class
var ApplicationsButton = new Lang.Class({
    Name: 'ApplicationsButton',
    Extends: PanelMenu.Button,

    // Initialize the menu
    _init: function(settings) {
        this.parent(1.0, null, false);

        this._settings = settings
        this.setMenu(new ApplicationsMenu(this.actor, 1.0, St.Side.TOP, this._settings));
        Main.panel.menuManager.addMenu(this.menu);
        this.actor.accessible_role = Atk.Role.LABEL;
        this._menuButton = new Widgets.MenuButton(Main.panel.actor);
        this.actor.add_child(this._menuButton.actor);
        this.actor.name = 'panelApplications';
        this.actor.connect('captured-event', Lang.bind(this, this._onCapturedEvent));
        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
        this._showingId = Main.overview.connect('showing', Lang.bind(this, function() {
            this.actor.add_accessible_state(Atk.StateType.CHECKED);
        }));
        this._hidingId = Main.overview.connect('hiding', Lang.bind(this, function() {
            this.actor.remove_accessible_state(Atk.StateType.CHECKED);
        }));
        this._menuKeybinder = new Keybinder.MenuKeybinder(Lang.bind(this, function() {
            this.menu.toggle();
        }));
        this._settings.connect('changed::super-hotkey', Lang.bind(this, this._updateKeybinding));
        this._updateKeybinding();
        this._desktopTarget = new Desktop.DesktopTarget();
        this._desktopTarget.connect('app-dropped', Lang.bind(this, function() {
            this.menu.close();
        }));
    },

    // Destroy the menu button
    _onDestroy: function() {
        Main.overview.disconnect(this._showingId);
        Main.overview.disconnect(this._hidingId);
        this._menuKeybinder.destroy();
        this._desktopTarget.destroy();
    },

    // Handle captured event
    _onCapturedEvent: function(actor, event) {
        if (event.type() == Clutter.EventType.BUTTON_PRESS) {
            if (!Main.overview.shouldToggleByCornerOrButton())
                return true;
        }
        return false;
    },

    _updateKeybinding: function() {
        let enableHotkey = this._settings.get_boolean('super-hotkey');
        if (enableHotkey) {
            this._menuKeybinder.enableHotKey();
        } else {
            this._menuKeybinder.disableHotKey();
        }
    }
});
